/*
 *  CsvJdbc - a JDBC driver for CSV files
 *  Copyright (C) 2001  Jonathan Ackerman
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.relique.jdbc.csv;

import java.sql.*;
import java.io.File;
import java.util.Enumeration;
import java.util.Vector;

/**
 * This class implements the Statement interface for the CsvJdbc driver.
 *
 * @author     Jonathan Ackerman
 * @author     Sander Brienen
 * @author     Michael Maraya
 * @created    25 November 2001
 * @version    $Id: CsvStatement.java,v 1.5 2002/08/24 23:38:53 mmaraya Exp $
 */

public class CsvStatement implements Statement
{
  private CsvConnection connection;
  private Vector resultSets = new Vector();


  /**
   *Constructor for the CsvStatement object
   *
   * @param  connection  Description of Parameter
   * @since
   */
  protected CsvStatement(CsvConnection connection)
  {
    DriverManager.println("CsvJdbc - CsvStatement() - connection=" + connection);
    this.connection = connection;
  }


  /**
   *Sets the maxFieldSize attribute of the CsvStatement object
   *
   * @param  p0                The new maxFieldSize value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public void setMaxFieldSize(int p0) throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Sets the maxRows attribute of the CsvStatement object
   *
   * @param  p0                The new maxRows value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public void setMaxRows(int p0) throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Sets the escapeProcessing attribute of the CsvStatement object
   *
   * @param  p0                The new escapeProcessing value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public void setEscapeProcessing(boolean p0) throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Sets the queryTimeout attribute of the CsvStatement object
   *
   * @param  p0                The new queryTimeout value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public void setQueryTimeout(int p0) throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Sets the cursorName attribute of the CsvStatement object
   *
   * @param  p0                The new cursorName value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public void setCursorName(String p0) throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Sets the fetchDirection attribute of the CsvStatement object
   *
   * @param  p0                The new fetchDirection value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public void setFetchDirection(int p0) throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Sets the fetchSize attribute of the CsvStatement object
   *
   * @param  p0                The new fetchSize value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public void setFetchSize(int p0) throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Gets the maxFieldSize attribute of the CsvStatement object
   *
   * @return                   The maxFieldSize value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public int getMaxFieldSize() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Gets the maxRows attribute of the CsvStatement object
   *
   * @return                   The maxRows value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public int getMaxRows() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Gets the queryTimeout attribute of the CsvStatement object
   *
   * @return                   The queryTimeout value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public int getQueryTimeout() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Gets the warnings attribute of the CsvStatement object
   *
   * @return                   The warnings value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public SQLWarning getWarnings() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Gets the resultSet attribute of the CsvStatement object
   *
   * @return                   The resultSet value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public ResultSet getResultSet() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Gets the updateCount attribute of the CsvStatement object
   *
   * @return                   The updateCount value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public int getUpdateCount() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Gets the moreResults attribute of the CsvStatement object
   *
   * @return                   The moreResults value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public boolean getMoreResults() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Gets the fetchDirection attribute of the CsvStatement object
   *
   * @return                   The fetchDirection value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public int getFetchDirection() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Gets the fetchSize attribute of the CsvStatement object
   *
   * @return                   The fetchSize value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public int getFetchSize() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Gets the resultSetConcurrency attribute of the CsvStatement object
   *
   * @return                   The resultSetConcurrency value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public int getResultSetConcurrency() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Gets the resultSetType attribute of the CsvStatement object
   *
   * @return                   The resultSetType value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public int getResultSetType() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Gets the connection attribute of the CsvStatement object
   *
   * @return                   The connection value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public Connection getConnection() throws SQLException
  {
    return connection;
  }


  /**
   *Description of the Method
   *
   * @param  sql               Description of Parameter
   * @return                   Description of the Returned Value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public ResultSet executeQuery(String sql) throws SQLException
  {
    DriverManager.println("CsvJdbc - CsvStatement:executeQuery() - sql= " + sql);
    SqlParser parser = new SqlParser();
    try
    {
      parser.parse(sql);
    }
    catch (Exception e)
    {
      throw new SQLException("Syntax Error. " + e.getMessage());
    }

//  if(parser.sqlType.equals(parser.SELECT)){

    String fileName = connection.getPath() + parser.getTableName() + connection.getExtension();
    File checkFile = new File(fileName);
    if (!checkFile.exists())
    {
      throw new SQLException("Cannot open data file '" + fileName + "'  !");
    }

    if (!checkFile.canRead())
    {
      throw new SQLException("Data file '" + fileName + "'  not readable !");
    }
    CsvReader reader;
    try
    {
      reader = new CsvReader(fileName, connection.getSeperator(), connection.isSuppressHeaders());
      String[] xxx = parser.getColumnNames();
      String[] yyy = reader.getColumnNames();
      boolean isOK = true;
      for(int i=0; i< xxx.length; i++) {
      out:
      for(int j=0; j< yyy.length; j++) {
         if(xxx[i].equalsIgnoreCase(yyy[j])) {
            isOK=true;
            break out;
          }
          else
            isOK=false;
        }
       if(!isOK)
        throw new SQLException("Column '" + xxx[i] + "' not found.");
      }
    }
    catch (Exception e)
    {
      throw new SQLException("Error reading data file. Message was: " + e);
    }
    CsvResultSet resultSet = new CsvResultSet(this, reader,
            parser.getTableName(), parser.getColumnNames(), parser.getWhereColumnNames(),
            parser.getWhereColumnValues());
    resultSets.add(resultSet);
    return resultSet;
//  }
//  	else
//		throw new SQLException("Support only SELECT SQL statements");

  }


  /**
   *Description of the Method
   *
   * @param  sql               Description of Parameter
   * @return                   Description of the Returned Value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public int executeUpdate(String sql) throws SQLException
  {
    DriverManager.println("CsvJdbc - CsvStatement:executeUpdate() - sql= " + sql);
    SqlParser parser = new SqlParser();
    try
    {
      parser.parse(sql);
    }
    catch (Exception e)
    {
      throw new SQLException("Syntax Error. " + e.getMessage());
    }
	  if(parser.sqlType.equals(parser.SELECT))
	  		throw new SQLException("Not supported SELECT statement - use executeQuery method");
	  else if (parser.sqlType.equals(parser.CREATE_TABLE)) {
	  			throw new SQLException("Not supported SELECT statement - use execute method");
	  }
	  else if (parser.sqlType.equals(parser.INSERT)) {
	  	   String fileName = connection.getPath() + parser.getTableName() + connection.getExtension();
    		File checkFile = new File(fileName);

    		if (!checkFile.exists())
    		{
      		throw new SQLException("Cannot open data file '" + fileName + "'  !");
    		}

    		if (!checkFile.canWrite())
    		{
      		throw new SQLException("Data file '" + fileName + "'  is read only !");
    		}
    	   CsvWriter writeCsv;
    		try
    		{
      		writeCsv = new CsvWriter(fileName, connection.getSeperator());
      		writeCsv.newLine(parser.columnNames, parser.columnValues);
                String[] xxx = parser.getColumnNames();
                String[] yyy = writeCsv.getColumnNames();
                boolean isOK = true;
                for(int i=0; i< xxx.length; i++) {
                  out:
                  for(int j=0; j< yyy.length; j++) {
                    if(xxx[i].equalsIgnoreCase(yyy[j])) {
                      isOK=true;
                      break out;
                    }
                  else
                    isOK=false;
                  }
                if(!isOK)
                  throw new SQLException("Column '" + xxx[i] + "' not found.");
              }


    		}
    		catch (Exception e)
    		{
      		throw new SQLException("Error reading data file. Message was: " + e);
    		}

	  }
	  else if (parser.sqlType.equals(parser.UPDATE)) {
	  	   String fileName = connection.getPath() + parser.getTableName() + connection.getExtension();
    		File checkFile = new File(fileName);

    		if (!checkFile.exists())
    		{
      		throw new SQLException("Cannot open data file '" + fileName + "'  !");
    		}

    		if (!checkFile.canWrite())
    		{
      		throw new SQLException("Data file '" + fileName + "'  is read only !");
    		}
    	   CsvWriter writeCsv;
    		try
    		{
      		writeCsv = new CsvWriter(fileName, connection.getSeperator());
      		writeCsv.updateField(parser.columnNames, parser.columnValues, parser.columnWhereNames, parser.columnWhereValues);
                String[] xxx = parser.getColumnNames();
                String[] yyy = writeCsv.getColumnNames();
                boolean isOK = true;
                for(int i=0; i< xxx.length; i++) {
                  out:
                  for(int j=0; j< yyy.length; j++) {
                    if(xxx[i].equalsIgnoreCase(yyy[j])) {
                      isOK=true;
                      break out;
                    }
                  else
                    isOK=false;
                }
                if(!isOK)
                  throw new SQLException("Column '" + xxx[i] + "' not found.");
             }

    		}
    		catch (Exception e)
    		{
      		throw new SQLException("Error reading data file. Message was: " + e);
    		}

		}
	  return 0;

  }


  /**
   * Releases this <code>Statement</code> object's database
   * and JDBC resources immediately instead of waiting for
   * this to happen when it is automatically closed.
   * It is generally good practice to release resources as soon as
   * you are finished with them to avoid tying up database
   * resources.
   * <P>
   * Calling the method <code>close</code> on a <code>Statement</code>
   * object that is already closed has no effect.
   * <P>
   * <B>Note:</B> A <code>Statement</code> object is automatically closed
   * when it is garbage collected. When a <code>Statement</code> object is
   * closed, its current <code>ResultSet</code> object, if one exists, is
   * also closed.
   *
   * @exception SQLException if a database access error occurs
   */
  public void close() throws SQLException {
      // close all result sets
      for(Enumeration i = resultSets.elements(); i.hasMoreElements(); ) {
          CsvResultSet resultSet = (CsvResultSet)i.nextElement();
          resultSet.close();
      }
  }


  /**
   *Description of the Method
   *
   * @exception  SQLException  Description of Exception
   * @since
   */
  public void cancel() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Description of the Method
   *
   * @exception  SQLException  Description of Exception
   * @since
   */
  public void clearWarnings() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Description of the Method
   *
   * @param  p0                Description of Parameter
   * @return                   Description of the Returned Value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public boolean execute(String sql) throws SQLException
  {

    SqlParser parser = new SqlParser();
    try
    {
      parser.parse(sql);
    }
    catch (Exception e)
    {
      throw new SQLException("Syntax Error. " + e.getMessage());
    }
	  if(parser.sqlType.equals(parser.SELECT))
	  		throw new SQLException("Not supported SELECT statement - use executeQuery method");
	  else if (parser.sqlType.equals(parser.INSERT))
	  		executeUpdate(sql);
 	  else if (parser.sqlType.equals(parser.CREATE_TABLE)) {
	  	   String fileName = connection.getPath() + parser.getTableName() + connection.getExtension();
    		File checkFile = new File(fileName);

    		if (checkFile.exists())
    		{
      		throw new SQLException("Data file '" + fileName + "'already exists  !");
    		}

    	   CsvWriter writeCsv;
    		try
    		{
      		writeCsv = new CsvWriter(fileName, connection.getSeperator());
      		writeCsv.createTable(parser.columnNames, fileName);
    		}
    		catch (Exception e)
    		{
      		throw new SQLException("Error reading data file. Message was: " + e);
    		}
    	}
		return true;

  }


  /**
   *Adds a feature to the Batch attribute of the CsvStatement object
   *
   * @param  p0                The feature to be added to the Batch attribute
   * @exception  SQLException  Description of Exception
   * @since
   */
  public void addBatch(String p0) throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Description of the Method
   *
   * @exception  SQLException  Description of Exception
   * @since
   */
  public void clearBatch() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }


  /**
   *Description of the Method
   *
   * @return                   Description of the Returned Value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public int[] executeBatch() throws SQLException
  {
    throw new SQLException("Not Supported !");
  }

  public boolean getMoreResults(int i)
            throws SQLException {
        throw new UnsupportedOperationException(
                "ResultSet.getMoreResult(int) unsupported");
   }

}

